#
#
# $Id: gtkext.rb,v 1.27 2005/12/31 13:34:45 ys Exp $
#
#

module Gtk

	class TextView
		def add_vikey_events(vadj)
			signal_connect('key_press_event') {|w, e|
				case e.keyval
				when Gdk::Keyval::GDK_space
					# vadj.set_value(vadj.value + vadj.page_size / 1.1)
					# set signal in Ci.rb.
				when Gdk::Keyval::GDK_BackSpace
					vadj.set_value(vadj.value - vadj.page_size / 1.1)
				when Gdk::Keyval::GDK_j
					if vadj.value < vadj.upper - vadj.page_size
						vadj.set_value(vadj.value + vadj.page_size / 10)
					end
				when Gdk::Keyval::GDK_k
					vadj.set_value(vadj.value - vadj.page_size / 10)
				when Gdk::Keyval::GDK_g
					vadj.set_value(vadj.lower)
				when Gdk::Keyval::GDK_G
					vadj.set_value(vadj.upper - vadj.page_size)
				end
			}
		end

		def add_mousescroll_events(vadj)
			signal_connect('button_press_event') {|w, e|
				case e.button
				when 5
					vadj.set_value(vadj.value + vadj.page_size / 4)
				when 4
					vadj.set_value(vadj.value - vadj.page_size / 4)
				end
			}
		end
	end


	class TreeView
		def add_vikey_events(vadj)
			signal_connect('key_press_event') {|w, e|
				begin
					case e.keyval
					when Gdk::Keyval::GDK_j
						path = selection.selected.path
						path.next!
						set_cursor(path, nil, nil)
					when Gdk::Keyval::GDK_k
						path = selection.selected.path
						path.prev!
						set_cursor(path, nil, nil)
					when Gdk::Keyval::GDK_g
						iter = selection.selected
						iter.first!
						set_cursor(iter.path, nil, nil)
					when Gdk::Keyval::GDK_G
						iter = selection.selected
						tmp = iter.dup
						first_iter = iter.dup
						first_iter.first!
						while true
							tmp.next!
							break if tmp == first_iter
							iter = tmp.dup
						end
						set_cursor(iter.path, nil, nil)
					end
				rescue
					set_cursor(model.iter_first.path, nil, nil)
				end
			}
		end
	end

end


module GtkExt

	def GtkExt.set_tooltip(widget, text)
		tooltip = Gtk::Tooltips.new
		tooltip.set_tip(widget, text, nil)
	end

	def GtkExt.parent_window(widget)
		if widget.kind_of? Gtk::Window
			return(widget)
		else
			return(parent_window(widget.parent))
		end
	end


	class AdjInfo
		def initialize(board, dat, adj, dir_name = false)
			@changing = false
			@board = board
			@dat = dat
			@adj = adj
			if dir_name
				@fn = "#{CFG::SYS.root}#{@board}/#{dir_name}/cache/#{@dat}.info"
			else
				@fn = "#{CFG::SYS.root}#{@board}/cache/#{@dat}.info"
			end
		end

		def get
			if File.exist?(@fn)
				return(IO.readlines(@fn).join.to_f)
			else
				return(0)
			end
		end

		def set
			fh = open(@fn, 'w')
			fh.write @adj.value.to_s
			fh.close
		end

		alias write set

		def exist?
			return(File.exist?(@fn))
		end
	end


	class Pixmap
	    def initialize(w, dir)
			@w = w
			@dir = dir
			@close = [
				"10 7 2 1",
				"  c black",
				".       c None s None",
				".  ....  .",
				"..  ..  ..",
				"...    ...",
				"....  ....",
				"...    ...",
				"..  ..  ..",
				".  ....  ."
			]
			@next = [
				"5 7 2 1",
				"  c black",
				".       c None s None",
				"  ...",
				".  ..",
				"..  .",
				"...  ",
				"..  .",
				".  ..",
				"  ..."
			]
			@prev = [
				"7 7 2 1",
				"  c black",
				".       c None s None",
				"....  .",
				"...  ..",
				"..  ...",
				".  ....",
				"..  ...",
				"...  ..",
				"....  ."
			]
			@all = [
				"10 7 2 1",
				"  c black",
				".       c None s None",
				"          ",
				"  ......  ",
				"  ......  ",
				"          ",
				"  ......  ",
				"  ......  ",
				"  ......  "
			]
			@bookmark2 = [
				"10 7 2 1",
				"  c black",
				".       c None s None",
				"          ",
				"  ......  ",
				"  ......  ",
				"          ",
				"  ......  ",
				"  ......  ",
				"          "
			]
			@update = [
				"10 7 2 1",
				"  c black",
				".       c None s None",
				"  ......  ",
				"  ......  ",
				"  ......  ",
				"  ......  ",
				"  ......  ",
				"  ......  ",
				"          "
			]
			@cache_plus = [
				"10 7 2 1",
				"  c black",
				".       c None s None",
				"          ",
				"  ........",
				"  ........",
				"  ........",
				"  ........",
				"  ........",
				"          "
			]
		end
	
		def get(name)
			if name == 'close'
				pix, mask = Gdk::Pixmap.create_from_xpm_d(@w.window, nil, @close)
			elsif name == 'next'
				pix, mask = Gdk::Pixmap.create_from_xpm_d(@w.window, nil, @next)
			elsif name == 'prev'
				pix, mask = Gdk::Pixmap.create_from_xpm_d(@w.window, nil, @prev)
			elsif name == 'all'
				pix, mask = Gdk::Pixmap.create_from_xpm_d(@w.window, nil, @all)
			elsif name == 'bookmark2'
				pix, mask = Gdk::Pixmap.create_from_xpm_d(@w.window, nil, @bookmark2)
			elsif name == 'update'
				pix, mask = Gdk::Pixmap.create_from_xpm_d(@w.window, nil, @update)
			elsif name == 'cache_plus'
				pix, mask = Gdk::Pixmap.create_from_xpm_d(@w.window, nil, @cache_plus)
			else
				pix, mask = Gdk::Pixmap.create_from_xpm(@w.window, nil,
					"#{@dir}#{name}.xpm")
			end
			return(Gtk::Image.new(pix, mask))
		end
	end


	class BBSMenuTree < Gtk::ScrolledWindow
		attr_reader :ctree, :vadj

		def initialize(item)
			@ctreein = Gtk::TreeStore.new(String)
			r = Gtk::CellRendererText.new
			@ctree = Gtk::TreeView.new(@ctreein)
			@ctree.append_column(Gtk::TreeViewColumn.new(item, r, {:text => 0}))

			@vadj = Gtk::Adjustment.new(0, 0, 0, 0, 0, 0)
			super(nil, @vadj)
			set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC)

			#@row = 0
	
			#@ctree.signal_connect('select_row') {|w, r, c, e| @row = r }
=begin
			@ctree.signal_connect('key_press_event') {|w, e|
				i = @vadj.upper / @ctree.rows  # AdjustmentʻCList1Ԥι⤵
				x = 0
				case e.keyval
				when Gdk::GDK_j
					x = @row + 1  # @rowϾselect_row signalǼ
					@ctree.select_row(x, 0)
					if x > (@vadj.value + @vadj.page_size + i) / i - 1.8  # MagicNumber. XD
						tmpVadj = @vadj.value + i
						if tmpVadj + @vadj.page_size < @vadj.upper
							@vadj.set_value(tmpVadj)
						else
							@vadj.set_value(@vadj.upper - @vadj.page_size)
						end
					end
				when Gdk::GDK_k
					x = @row - 1  # @rowϾselect_row signalǼ
					@ctree.select_row(x, 0)
					if x < (@vadj.value - i) / i + 0.8  # MagicNumber. XD
						tmpVadj = @vadj.value - @vadj.upper / @ctree.rows
						if tmpVadj + @vadj.page_size > @vadj.lower
							@vadj.set_value(tmpVadj)
						else
							@vadj.set_value(@vadj.lower)
						end
					end
				when Gdk::GDK_l
					node = ''
					@ctree.each_selection {|n| node = n }
					@ctree.toggle_expansion(node) unless @ctree.get_node_info(node)[6]
				when Gdk::GDK_g
					@ctree.select_row(0, 0)
					@vadj.set_value(@vadj.lower)
				when Gdk::GDK_G
					@ctree.select_row(@ctree.rows - 1, 0)
					@vadj.set_value(@vadj.upper - @vadj.page_size)
				end
			}
=end
			self.add(@ctree)
		end
		
		def child
			@ctree
		end
	end
end
