#
#
# $Id: gtkmisc.rb,v 1.22 2005/12/31 13:36:05 ys Exp $
#
#

module GtkExt
	
	class Windmill < Gtk::Button  #=> Thread Manager for C.i.
		SLEEP_TIME = 1
		MAX_JOB = 2

		def initialize
			@ab = Pix.get('about')
			@wind = []
			['load', 'load+1', 'load+2', 'load+1',
			 'load', 'load-1', 'load-2', 'load-1'].each {|i| @wind << Pix.get(i)}
			super()
			add @ab
			@thr = Thread.fork { Thread.exit }
			@q = []
		end
	
		def start(purpose = '')
			if @q.include?(purpose)
				return(false) #=> already.
			else
				if @q.empty?
					@thr = Thread.fork {
						show_all
						while true
							@wind.each {|w| remove child; add(w).show_all ; sleep 0.5 }
						end
					}
				elsif q.size >= MAX_JOB
					while q.size < MAX_JOB
						sleep(SLEEP_TIME)
					end
				end
				@q << purpose
				return(true) #=> OK.
			end
		end
	
		def stop(purpose = '')
			@q.delete(purpose)
			if @q.empty?
				@thr.kill if @thr.alive?
				remove child
				add @ab
			end
		end
	
		def kill
			@thr.kill if @thr.alive?
				remove child
				add @ab
			@q.clear
		end
	
		def q
			return(@q)
		end
	end


	class UrlView < Gtk::Combo  # for C.i.
		include Searched

		def initialize
			super()
			entry.set_editable(true)
			disable_activate
			set_use_arrows_always(true)
			set_case_sensitive(false)
			set_popdown_strings([''])
		end
	
		def set_board(board)
			x = []
			get_searched(board).each {|i| x << Uconv.euctou8(i)}
			set_popdown_strings(x)
		end
	
		def add_str(board, str)
			x = entry.text
			add_searched(board, Uconv.u8toeuc(str)) if str != ""
			set_board(board)
			entry.set_text(x)
		end
	end


	class TopButtons < Gtk::HBox # for C.i.
		attr_reader :select, :bookmark, :reload, :cancel, :find, :exit, :followup
		attr_reader :urlview, :windmill

		def initialize

			@select = Gtk::ToggleButton.new.add Pix.get('boardselect')
				GtkExt.set_tooltip(@select, CFG::NLS.ttg_select)
			@bookmark = Gtk::ToggleButton.new.add Pix.get('bookmark')
				GtkExt.set_tooltip(@bookmark, CFG::NLS.ttg_bookmark)
			@reload = Gtk::Button.new.add Pix.get('refresh_blue')
				GtkExt.set_tooltip(@reload, CFG::NLS.ttg_reload)
			@cancel = Gtk::Button.new.add Pix.get('cancel')
				GtkExt.set_tooltip(@cancel, CFG::NLS.ttg_cancel)
			@find = Gtk::Button.new.add Pix.get('search')
				GtkExt.set_tooltip(@find, CFG::NLS.ttg_find)
			@followup = Gtk::Button.new.add Pix.get('edit')
				GtkExt.set_tooltip(@followup, CFG::NLS.ttg_followup)

			@urlview = UrlView.new
			@windmill = Windmill.new

			super()
			[@select, @reload, @cancel, @followup].each {|w|
				pack_start w, false ; w.relief = Gtk::RELIEF_NONE
			}
			pack_start @urlview, true
			[@find, @windmill].each {|w|
				pack_start w, false ; w.relief = Gtk::RELIEF_NONE
			}
		end
	end


	class About < Gtk::Window
		def initialize
			super()
	
			v_str =  "#{CiLongString.sub('version', "\nversion")}\n\n"
			v_str << "(C)2005 H.Miyamoto (aka \"Yuukis\")\n"
			v_str << "http://wids.net/lab/Ci.html\n"
			v_str << "Ys@PixyGarden.net\n"
			v_label = Gtk::Label.new(v_str)
			v_label.set_justify(Gtk::JUSTIFY_CENTER)
	
			close = Gtk::Button.new("\s"*5 + "OK" + "\s"*5)
			close.signal_connect('clicked') { destroy }
			bbox = Gtk::HBox.new
			bbox.pack_start Gtk::Label.new('')
			bbox.pack_start close, false
	
			xbox = Gtk::VBox.new
			xbox.pack_start v_label
			xbox.pack_start bbox, false
	
			box = Gtk::VBox.new(false, 10)
			box.border_width = 8
			box.pack_start Pix.get('logo_epsilon')
			box.pack_start xbox
			add box
			set_title('About C.i.')
			set_focus(close)
			signal_connect('delete_event') { destroy }
			self
		end
	end
	

	class ConfigPop < Gtk::Menu
	
		attr_reader :update_bbsmenu, :stype, :hide, :config, :about
	
		def initialize
			super()
	
			@update_bbsmenu = Gtk::MenuItem.new(CFG::NLS.ttc_update)
			@stype = Gtk::MenuItem.new(CFG::NLS.ttc_stype)
			@hide = Gtk::MenuItem.new(CFG::NLS.ttc_hide)
			@config = Gtk::MenuItem.new(CFG::NLS.ttc_config)
			@about = Gtk::MenuItem.new(CFG::NLS.ttc_about)
	
			append @update_bbsmenu
			append @stype
			append @hide
			append @config
			append @about
	
			show_all
		end
	end


	module Menus  # for ThreadFrame AA / Filter
		def Menus.init(tooltip, name)
			button = Gtk::MenuBar.new
			GtkExt.set_tooltip(button, tooltip)
			menu_root = Gtk::MenuItem.new.add Gtk::Label.new(name)
			button.append(menu_root)
			menu = Gtk::Menu.new
			menu_root.set_submenu(menu)
			return([button, menu_root, menu])
		end
	end


	class FollowPop < Gtk::Menu  # for ThreadFrame
		def initialize(str)
			super()
			fs = Gtk::MenuItem.new(str)
			fs.child.set_justify(Gtk::JUSTIFY_LEFT)
			fs.signal_connect('button_press_event'){
				popdown
			}
	
			append Gtk::MenuItem.new
			append fs
			append Gtk::MenuItem.new
	
			show_all
		end
	end


	class RightMenu < Gtk::Menu  # for SubjectFrame / ThreadFrame
		attr_reader :title, :browser, :bookmark, :abone

		def initialize(title)
			super()
			@title = Gtk::MenuItem.new(title)
			self.append @title
			self.append Gtk::MenuItem.new
			@browser = Gtk::MenuItem.new('Open by Browser')
			self.append @browser
			@bookmark = Gtk::MenuItem.new('')
			self.append @bookmark
			@abone = Gtk::MenuItem.new('')
			self.append @abone
	
			self.show_all
		end
	end
end
